@@ -35,8 +35,9 @@
 #endif
 
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-account.h>
 
+#include <telepathy-glib/account.h>
+#include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/connection-manager.h>
 #include <telepathy-glib/util.h>
 #include <dbus/dbus-protocol.h>
@@ -70,6 +71,11 @@ typedef struct {
   /* An EmpathyAccountWidget can be used to either create an account or
    * modify it. When we are creating an account, this member is set to TRUE */
   gboolean creating_account;
+  /* If we are creating a new account, this member is set to TRUE once the
+   * account has been created */
+  gboolean account_created;
+
+  TpAccountManager *account_manager;
 
   gboolean dispose_run;
 } EmpathyAccountWidgetPriv;
@@ -291,7 +297,46 @@ account_widget_jabber_ssl_toggled_cb (GtkWidget *checkbutton_ssl,
 }
 
 static void
-account_widget_setup_widget (EmpathyAccountWidget *self,
+account_widget_combobox_changed_cb (GtkWidget *widget,
+    EmpathyAccountWidget *self)
+{
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+  const gchar *value;
+  const GValue *v;
+  const gchar *default_value = NULL;
+  const gchar *param_name;
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
+  if (!gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter))
+    return;
+
+  model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
+  /* the param value is stored in the first column */
+  gtk_tree_model_get (model, &iter, 0, &value, -1);
+
+  param_name = g_object_get_data (G_OBJECT (widget), "param_name");
+
+  v = empathy_account_settings_get_default (priv->settings, param_name);
+  if (v != NULL)
+    default_value = g_value_get_string (v);
+
+  if (!tp_strdiff (value, default_value))
+    {
+      DEBUG ("Unset %s and restore to %s", param_name, default_value);
+      empathy_account_settings_unset (priv->settings, param_name);
+    }
+  else
+    {
+      DEBUG ("Setting %s to %s", param_name, value);
+      empathy_account_settings_set_string (priv->settings, param_name, value);
+    }
+
+  account_widget_handle_control_buttons_sensitivity (self);
+}
+
+void
+empathy_account_widget_setup_widget (EmpathyAccountWidget *self,
     GtkWidget *widget,
     const gchar *param_name)
 {
@@ -366,6 +411,41 @@ account_widget_setup_widget (EmpathyAccountWidget *self,
           G_CALLBACK (account_widget_checkbutton_toggled_cb),
           self);
     }
+  else if (GTK_IS_COMBO_BOX (widget))
+    {
+      /* The combo box's model has to contain the param value in its first
+       * column (as a string) */
+      const gchar *str;
+      GtkTreeModel *model;
+      GtkTreeIter iter;
+      gboolean valid;
+
+      str = empathy_account_settings_get_string (priv->settings, param_name);
+      model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
+
+      valid = gtk_tree_model_get_iter_first (model, &iter);
+      while (valid)
+        {
+          gchar *name;
+
+          gtk_tree_model_get (model, &iter, 0, &name, -1);
+          if (!tp_strdiff (name, str))
+            {
+              gtk_combo_box_set_active_iter (GTK_COMBO_BOX (widget), &iter);
+              valid = FALSE;
+            }
+          else
+            {
+              valid = gtk_tree_model_iter_next (model, &iter);
+            }
+
+          g_free (name);
+        }
+
+      g_signal_connect (widget, "changed",
+          G_CALLBACK (account_widget_combobox_changed_cb),
+          self);
+    }
   else
     {
       DEBUG ("Unknown type of widget for param %s", param_name);
@@ -526,7 +606,7 @@ accounts_widget_generic_setup (EmpathyAccountWidget *self,
         }
 
       if (widget)
-        account_widget_setup_widget (self, widget, param->name);
+        empathy_account_widget_setup_widget (self, widget, param->name);
 
       g_free (param_name_formatted);
     }
@@ -553,7 +633,8 @@ account_widget_handle_params_valist (EmpathyAccountWidget *self,
           continue;
         }
 
-      account_widget_setup_widget (self, GTK_WIDGET (object), param_name);
+      empathy_account_widget_setup_widget (self, GTK_WIDGET (object),
+          param_name);
     }
 }
 
@@ -570,10 +651,11 @@ account_widget_account_enabled_cb (GObject *source_object,
     gpointer user_data)
 {
   GError *error = NULL;
-  EmpathyAccount *account = EMPATHY_ACCOUNT (source_object);
+  TpAccount *account = TP_ACCOUNT (source_object);
   EmpathyAccountWidget *widget = EMPATHY_ACCOUNT_WIDGET (user_data);
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
 
-  empathy_account_set_enabled_finish (account, res, &error);
+  tp_account_set_enabled_finish (account, res, &error);
 
   if (error != NULL)
     {
@@ -582,6 +664,7 @@ account_widget_account_enabled_cb (GObject *source_object,
     }
   else
     {
+      priv->account_created = TRUE;
       g_signal_emit (widget, signals[ACCOUNT_CREATED], 0);
     }
 }
@@ -592,7 +675,7 @@ account_widget_applied_cb (GObject *source_object,
     gpointer user_data)
 {
   GError *error = NULL;
-  EmpathyAccount *account;
+  TpAccount *account;
   EmpathyAccountSettings *settings = EMPATHY_ACCOUNT_SETTINGS (source_object);
   EmpathyAccountWidget *widget = EMPATHY_ACCOUNT_WIDGET (user_data);
   EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
@@ -613,7 +696,7 @@ account_widget_applied_cb (GObject *source_object,
       if (priv->creating_account)
         {
           /* By default, when an account is created, we enable it. */
-          empathy_account_set_enabled_async (account, TRUE,
+          tp_account_set_enabled_async (account, TRUE,
               account_widget_account_enabled_cb, widget);
         }
       else if (priv->enabled_checkbox != NULL)
@@ -629,12 +712,12 @@ account_widget_applied_cb (GObject *source_object,
                 NBTK_GTK_LIGHT_SWITCH (priv->enabled_checkbox));
 #endif
 
-          if (empathy_account_is_enabled (account) && enabled_checked)
+          if (tp_account_is_enabled (account) && enabled_checked)
             {
               /* After having applied changes to a user account, we
                * automatically reconnect it. This is done so the new
                * information entered by the user is validated on the server. */
-              empathy_account_reconnect_async (account, NULL, NULL);
+              tp_account_reconnect_async (account, NULL, NULL);
             }
         }
     }
@@ -726,7 +809,7 @@ account_widget_build_salut (EmpathyAccountWidget *self,
       "entry_jid", "jid",
       NULL);
 
-  self->ui_details->default_focus = g_strdup ("entry_nickname");
+  self->ui_details->default_focus = g_strdup ("entry_first_name");
 }
 
 static void
@@ -1047,13 +1130,13 @@ account_widget_destroy_cb (GtkWidget *widget,
 }
 
 static void
-empathy_account_widget_enabled_cb (EmpathyAccount *account,
+empathy_account_widget_enabled_cb (TpAccount *account,
       GParamSpec *spec,
       gpointer user_data)
 {
   EmpathyAccountWidget *widget = EMPATHY_ACCOUNT_WIDGET (user_data);
   EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
-  gboolean enabled = empathy_account_is_enabled (account);
+  gboolean enabled = tp_account_is_enabled (account);
 
   if (priv->enabled_checkbox != NULL)
     {
@@ -1079,7 +1162,7 @@ account_widget_switch_flipped_cb (NbtkGtkLightSwitch *sw,
 #endif /* HAVE_MOBLIN */
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (user_data);
-  EmpathyAccount *account;
+  TpAccount *account;
 #ifndef HAVE_MOBLIN
   gboolean state;
 
@@ -1089,7 +1172,7 @@ account_widget_switch_flipped_cb (NbtkGtkLightSwitch *sw,
   account = empathy_account_settings_get_account (priv->settings);
 
   /* Enable the account according to the value of the "Enabled" checkbox */
-  empathy_account_set_enabled_async (account, state, NULL, NULL);
+  tp_account_set_enabled_async (account, state, NULL, NULL);
 }
 
 static void
@@ -1144,6 +1227,61 @@ do_get_property (GObject *object,
     }
 }
 
+static void
+presence_changed_cb (TpAccountManager *manager,
+    TpConnectionPresenceType state,
+    const gchar *status,
+    const gchar *message,
+    EmpathyAccountWidget *self)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+
+  if (state > TP_CONNECTION_PRESENCE_TYPE_OFFLINE)
+    {
+      /* We are online, display a Login button */
+      GtkWidget *image;
+
+      gtk_button_set_use_stock (GTK_BUTTON (priv->apply_button), FALSE);
+      gtk_button_set_label (GTK_BUTTON (priv->apply_button), _("L_og in"));
+
+      image = gtk_image_new_from_stock (GTK_STOCK_CONNECT,
+          GTK_ICON_SIZE_BUTTON);
+      gtk_button_set_image (GTK_BUTTON (priv->apply_button), image);
+    }
+  else
+    {
+      /* We are offline, display a Save button */
+      gtk_button_set_image (GTK_BUTTON (priv->apply_button), NULL);
+      gtk_button_set_use_stock (GTK_BUTTON (priv->apply_button), TRUE);
+      gtk_button_set_label (GTK_BUTTON (priv->apply_button), GTK_STOCK_SAVE);
+    }
+}
+
+static void
+account_manager_ready_cb (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  EmpathyAccountWidget *self = EMPATHY_ACCOUNT_WIDGET (user_data);
+  TpAccountManager *account_manager = TP_ACCOUNT_MANAGER (source_object);
+  GError *error = NULL;
+  TpConnectionPresenceType state;
+
+  if (!tp_account_manager_prepare_finish (account_manager, result, &error))
+    {
+      DEBUG ("Failed to prepare account manager: %s", error->message);
+      g_error_free (error);
+      return;
+    }
+
+  state = tp_account_manager_get_most_available_presence (account_manager, NULL,
+      NULL);
+
+  /* simulate a presence change so the apply button will be changed
+   * if needed */
+  presence_changed_cb (account_manager, state, NULL, NULL, self);
+}
+
 #define WIDGET(cm, proto) \
   { #cm, #proto, "empathy-account-widget-"#proto".ui", \
     account_widget_build_##proto }
@@ -1153,9 +1291,9 @@ do_constructed (GObject *obj)
 {
   EmpathyAccountWidget *self = EMPATHY_ACCOUNT_WIDGET (obj);
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
-  EmpathyAccount *account;
+  TpAccount *account;
   const gchar *protocol, *cm_name;
-  int i = 0;
+  guint i = 0;
   struct {
     const gchar *cm_name;
     const gchar *protocol;
@@ -1242,15 +1380,27 @@ do_constructed (GObject *obj)
       GtkWidget *hbox = gtk_hbox_new (TRUE, 3);
 
       priv->cancel_button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
-      priv->apply_button = gtk_button_new_from_stock (
-        priv->creating_account ? GTK_STOCK_CONNECT : GTK_STOCK_APPLY);
 
-#ifdef HAVE_MOBLIN
       if (priv->creating_account)
-	/* Translators: this is used only when built on a moblin platform */
-	gtk_button_set_label (GTK_BUTTON (priv->apply_button),
-            _("L_og in"));
-#endif
+        {
+          priv->account_manager = tp_account_manager_dup ();
+
+          empathy_signal_connect_weak (priv->account_manager,
+              "most-available-presence-changed",
+              G_CALLBACK (presence_changed_cb), obj);
+
+          tp_account_manager_prepare_async (priv->account_manager, NULL,
+              account_manager_ready_cb, self);
+
+          /* Assumre we are offline, display a Save button. We'll update
+           * it once the account manager is ready if needed */
+          priv->apply_button = gtk_button_new_from_stock (GTK_STOCK_SAVE);
+        }
+      else
+        {
+          /* We are editing an existing account, display an Apply button */
+          priv->apply_button = gtk_button_new_from_stock (GTK_STOCK_APPLY);
+        }
 
       gtk_box_pack_end (GTK_BOX (hbox), priv->apply_button, TRUE,
           TRUE, 3);
@@ -1295,7 +1445,7 @@ do_constructed (GObject *obj)
       guint nb_rows, nb_columns;
       gboolean is_enabled;
 
-      is_enabled = empathy_account_is_enabled (account);
+      is_enabled = tp_account_is_enabled (account);
 
 #ifndef HAVE_MOBLIN
       priv->enabled_checkbox =
@@ -1373,7 +1523,7 @@ do_dispose (GObject *obj)
 
   if (priv->settings != NULL)
     {
-      EmpathyAccount *account;
+      TpAccount *account;
       account = empathy_account_settings_get_account (priv->settings);
 
       if (account != NULL)
@@ -1386,6 +1536,12 @@ do_dispose (GObject *obj)
       priv->settings = NULL;
     }
 
+  if (priv->account_manager != NULL)
+    {
+      g_object_unref (priv->account_manager);
+      priv->account_manager = NULL;
+    }
+
   if (G_OBJECT_CLASS (empathy_account_widget_parent_class)->dispose != NULL)
     G_OBJECT_CLASS (empathy_account_widget_parent_class)->dispose (obj);
 }
@@ -1494,6 +1650,11 @@ empathy_account_widget_contains_pending_changes (EmpathyAccountWidget *widget)
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
 
+  if (priv->creating_account && !priv->account_created)
+    /* We always want to warn the user if he's in the process of creating a
+     * new account which hasn't been actually created yet. */
+    return TRUE;
+
   return priv->contains_pending_changes;
 }
 
@@ -1532,3 +1693,56 @@ empathy_account_widget_new_for_protocol (EmpathyAccountSettings *settings,
 
   return self;
 }
+
+gchar *
+empathy_account_widget_get_default_display_name (EmpathyAccountWidget *self)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  const gchar *login_id;
+  const gchar *protocol, *p;
+  gchar *default_display_name;
+
+  login_id = empathy_account_settings_get_string (priv->settings, "account");
+  protocol = empathy_account_settings_get_protocol (priv->settings);
+
+  if (login_id != NULL)
+    {
+      /* TODO: this should be done in empathy-account-widget-irc */
+      if (!tp_strdiff (protocol, "irc"))
+        {
+          const gchar* server;
+          server = empathy_account_settings_get_string (priv->settings,
+              "server");
+
+          /* To translators: The first parameter is the login id and the
+           * second one is the server. The resulting string will be something
+           * like: "MyUserName on chat.freenode.net".
+           * You should reverse the order of these arguments if the
+           * server should come before the login id in your locale.*/
+          default_display_name = g_strdup_printf (_("%1$s on %2$s"),
+              login_id, server);
+        }
+      else
+        {
+          default_display_name = g_strdup (login_id);
+        }
+
+      return default_display_name;
+    }
+
+  if ((p = empathy_protocol_name_to_display_name (protocol)) != NULL)
+    protocol = p;
+
+  if (protocol != NULL)
+    {
+      /* To translators: The parameter is the protocol name. The resulting
+       * string will be something like: "Jabber Account" */
+      default_display_name = g_strdup_printf (_("%s Account"), protocol);
+    }
+  else
+    {
+      default_display_name = g_strdup (_("New account"));
+    }
+
+  return default_display_name;
+}
