@@ -32,6 +32,7 @@
 #include <champlain-gtk/champlain-gtk.h>
 #endif
 
+#include <telepathy-glib/account.h>
 #include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-tp-contact-factory.h>
@@ -40,7 +41,6 @@
 #include <libempathy/empathy-location.h>
 #include <libempathy/empathy-time.h>
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-account.h>
 
 #include "empathy-contact-widget.h"
 #include "empathy-account-chooser.h"
@@ -496,19 +496,19 @@ save_avatar_menu_activate_cb (GtkWidget *widget,
       if (!empathy_avatar_save_to_file (avatar, filename, &error))
         {
           /* Save error */
-          GtkWidget *dialog;
+          GtkWidget *error_dialog;
 
-          dialog = gtk_message_dialog_new (NULL, 0,
+          error_dialog = gtk_message_dialog_new (NULL, 0,
               GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
               _("Unable to save avatar"));
 
           gtk_message_dialog_format_secondary_text (
-              GTK_MESSAGE_DIALOG (dialog), "%s", error->message);
+              GTK_MESSAGE_DIALOG (error_dialog), "%s", error->message);
 
-          g_signal_connect (dialog, "response",
+          g_signal_connect (error_dialog, "response",
               G_CALLBACK (gtk_widget_destroy), NULL);
 
-          gtk_window_present (GTK_WINDOW (dialog));
+          gtk_window_present (GTK_WINDOW (error_dialog));
 
           g_clear_error (&error);
         }
@@ -600,6 +600,10 @@ contact_widget_contact_setup (EmpathyContactWidget *information)
                                 PANGO_WRAP_WORD_CHAR);
   gtk_label_set_line_wrap (GTK_LABEL (information->label_status),
                            TRUE);
+
+  if (!(information->flags & EMPATHY_CONTACT_WIDGET_FOR_TOOLTIP))
+    gtk_label_set_selectable (GTK_LABEL (information->label_status), TRUE);
+
   gtk_box_pack_start (GTK_BOX (information->hbox_presence),
         information->label_status, TRUE, TRUE, 0);
   gtk_widget_show (information->label_status);
@@ -714,7 +718,7 @@ contact_widget_contact_setup (EmpathyContactWidget *information)
 static void
 contact_widget_contact_update (EmpathyContactWidget *information)
 {
-  EmpathyAccount *account = NULL;
+  TpAccount *account = NULL;
   const gchar *id = NULL;
 
   /* Connect and get info from new contact */
@@ -754,7 +758,7 @@ contact_widget_contact_update (EmpathyContactWidget *information)
         {
           const gchar *name;
 
-          name = empathy_account_get_display_name (account);
+          name = tp_account_get_display_name (account);
           gtk_label_set_label (GTK_LABEL (information->widget_account), name);
         }
     }
@@ -1316,8 +1320,22 @@ contact_widget_location_update (EmpathyContactWidget *information)
   gboolean has_position = TRUE;
   GtkWidget *label;
   guint row = 0;
-  GHashTableIter iter;
-  gpointer key, pvalue;
+  static const gchar* ordered_geolocation_keys[] = {
+    EMPATHY_LOCATION_TEXT,
+    EMPATHY_LOCATION_URI,
+    EMPATHY_LOCATION_DESCRIPTION,
+    EMPATHY_LOCATION_BUILDING,
+    EMPATHY_LOCATION_FLOOR,
+    EMPATHY_LOCATION_ROOM,
+    EMPATHY_LOCATION_STREET,
+    EMPATHY_LOCATION_AREA,
+    EMPATHY_LOCATION_LOCALITY,
+    EMPATHY_LOCATION_REGION,
+    EMPATHY_LOCATION_COUNTRY,
+    NULL
+  };
+  int i;
+  const gchar *skey;
 
   if (!(information->flags & EMPATHY_CONTACT_WIDGET_SHOW_LOCATION))
     {
@@ -1352,12 +1370,12 @@ contact_widget_location_update (EmpathyContactWidget *information)
       gchar *user_date;
       gchar *text;
       gint64 stamp;
-      time_t time;
+      time_t time_;
 
       stamp = g_value_get_int64 (value);
-      time = stamp;
+      time_ = stamp;
 
-      user_date = empathy_time_to_string_relative (time);
+      user_date = empathy_time_to_string_relative (time_);
 
       text = g_strconcat ( _("<b>Location</b>, "), user_date, NULL);
       gtk_label_set_markup (GTK_LABEL (information->label_location), text);
@@ -1375,18 +1393,18 @@ contact_widget_location_update (EmpathyContactWidget *information)
   gtk_box_pack_start (GTK_BOX (information->subvbox_location),
       information->table_location, FALSE, FALSE, 5);
 
-  g_hash_table_iter_init (&iter, location);
-  while (g_hash_table_iter_next (&iter, &key, &pvalue))
+
+  for (i = 0; (skey = ordered_geolocation_keys[i]); i++)
     {
-      const gchar *skey;
       const gchar* user_label;
       GValue *gvalue;
       char *svalue = NULL;
 
-      skey = (const gchar *) key;
+      gvalue = g_hash_table_lookup (location, (gpointer) skey);
+      if (gvalue == NULL)
+        continue;
 
       user_label = location_key_to_label (skey);
-      gvalue = (GValue *) pvalue;
 
       label = gtk_label_new (user_label);
       gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
@@ -1406,10 +1424,10 @@ contact_widget_location_update (EmpathyContactWidget *information)
         }
       else if (G_VALUE_TYPE (gvalue) == G_TYPE_INT64)
         {
-          time_t time;
+          time_t time_;
 
-          time = g_value_get_int64 (value);
-          svalue = empathy_time_to_string_utc (time, _("%B %e, %Y at %R UTC"));
+          time_ = g_value_get_int64 (value);
+          svalue = empathy_time_to_string_utc (time_, _("%B %e, %Y at %R UTC"));
         }
 
       if (svalue != NULL)
@@ -1419,12 +1437,21 @@ contact_widget_location_update (EmpathyContactWidget *information)
               label, 1, 2, row, row + 1);
           gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
           gtk_widget_show (label);
+
+          if (!(information->flags & EMPATHY_CONTACT_WIDGET_FOR_TOOLTIP))
+            gtk_label_set_selectable (GTK_LABEL (label), TRUE);
         }
 
       g_free (svalue);
       row++;
     }
 
+  if (row == 0)
+    {
+      gtk_widget_hide (information->vbox_location);
+      return;
+    }
+
   gtk_widget_show (information->table_location);
 
 #if HAVE_LIBCHAMPLAIN
