@@ -32,10 +32,10 @@
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
 
+#include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
 
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-connection-managers.h>
 #include <libempathy-gtk/empathy-ui-utils.h>
 
@@ -61,6 +61,10 @@
  * unsaved changes */
 #define PENDING_CHANGES_QUESTION_PRIMARY_TEXT \
   _("There are unsaved modification regarding your %s account.")
+/* The primary text of the dialog shown to the user when he is about to lose
+ * an unsaved new account */
+#define UNSAVED_NEW_ACCOUNT_QUESTION_PRIMARY_TEXT \
+  _("Your new account has not been saved yet.")
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyAccountsDialog)
 G_DEFINE_TYPE (EmpathyAccountsDialog, empathy_accounts_dialog, G_TYPE_OBJECT);
@@ -107,11 +111,11 @@ typedef struct {
   gulong  settings_ready_id;
   EmpathyAccountSettings *settings_ready;
 
-  EmpathyAccountManager *account_manager;
+  TpAccountManager *account_manager;
   EmpathyConnectionManagers *cms;
 
   GtkWindow *parent_window;
-  EmpathyAccount *initial_selection;
+  TpAccount *initial_selection;
 
   /* Those are needed when changing the selected row. When a user selects
    * another account and there are unsaved changes on the currently selected
@@ -146,9 +150,6 @@ static gboolean accounts_dialog_get_settings_iter (
 
 static void accounts_dialog_model_select_first (EmpathyAccountsDialog *dialog);
 
-static void accounts_dialog_update (EmpathyAccountsDialog *dialog,
-    EmpathyAccountSettings *settings);
-
 static void accounts_dialog_update_settings (EmpathyAccountsDialog *dialog,
     EmpathyAccountSettings *settings);
 
@@ -175,7 +176,7 @@ empathy_account_dialog_widget_cancelled_cb (
   GtkTreeSelection *selection;
   GtkTreeIter iter;
   EmpathyAccountSettings *settings;
-  EmpathyAccount *account;
+  TpAccount *account;
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
   view = GTK_TREE_VIEW (priv->treeview);
@@ -207,53 +208,6 @@ empathy_account_dialog_widget_cancelled_cb (
     g_object_unref (settings);
 }
 
-static gchar *
-get_default_display_name (EmpathyAccountSettings *settings)
-{
-  const gchar *login_id;
-  const gchar *protocol, *p;
-  gchar *default_display_name;
-
-  login_id = empathy_account_settings_get_string (settings, "account");
-  protocol = empathy_account_settings_get_protocol (settings);
-
-  if ((p = empathy_protocol_name_to_display_name (protocol)) != NULL)
-    protocol = p;
-
-  if (login_id != NULL)
-    {
-      if (!tp_strdiff (protocol, "irc"))
-        {
-          const gchar* server;
-          server = empathy_account_settings_get_string (settings, "server");
-
-          /* To translators: The first parameter is the login id and the
-           * second one is the server. The resulting string will be something
-           * like: "MyUserName on chat.freenode.net".
-           * You should reverse the order of these arguments if the
-           * server should come before the login id in your locale.*/
-          default_display_name = g_strdup_printf (_("%1$s on %2$s"),
-              login_id, server);
-        }
-      else
-        {
-          default_display_name = g_strdup (login_id);
-        }
-    }
-  else if (protocol != NULL)
-    {
-      /* To translators: The parameter is the protocol name. The resulting
-       * string will be something like: "Jabber Account" */
-      default_display_name = g_strdup_printf (_("%s Account"), protocol);
-    }
-  else
-    {
-      default_display_name = g_strdup (_("New account"));
-    }
-
-  return default_display_name;
-}
-
 static void
 empathy_account_dialog_account_created_cb (EmpathyAccountWidget *widget_object,
     EmpathyAccountsDialog *dialog)
@@ -262,7 +216,8 @@ empathy_account_dialog_account_created_cb (EmpathyAccountWidget *widget_object,
   EmpathyAccountSettings *settings =
       accounts_dialog_model_get_selected_settings (dialog);
 
-  display_name = get_default_display_name (settings);
+  display_name = empathy_account_widget_get_default_display_name (
+      widget_object);
 
   empathy_account_settings_set_display_name_async (settings,
       display_name, NULL, NULL);
@@ -354,7 +309,7 @@ accounts_dialog_model_select_first (EmpathyAccountsDialog *dialog)
 
 static gboolean
 accounts_dialog_has_pending_change (EmpathyAccountsDialog *dialog,
-    EmpathyAccount **account)
+    TpAccount **account)
 {
   GtkTreeIter iter;
   GtkTreeModel *model;
@@ -366,7 +321,7 @@ accounts_dialog_has_pending_change (EmpathyAccountsDialog *dialog,
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
     gtk_tree_model_get (model, &iter, COL_ACCOUNT_POINTER, account, -1);
 
-  return *account != NULL && priv->setting_widget_object != NULL
+  return priv->setting_widget_object != NULL
       && empathy_account_widget_contains_pending_changes (
           priv->setting_widget_object);
 }
@@ -498,17 +453,31 @@ accounts_dialog_add_pending_changes_response_cb (GtkDialog *message_dialog,
     }
 }
 
+static gchar *
+get_dialog_primary_text (TpAccount *account)
+{
+  if (account != NULL)
+    {
+      /* Existing account */
+      return g_strdup_printf (PENDING_CHANGES_QUESTION_PRIMARY_TEXT,
+          tp_account_get_display_name (account));
+    }
+  else
+    {
+      /* Newly created account */
+      return g_strdup (UNSAVED_NEW_ACCOUNT_QUESTION_PRIMARY_TEXT);
+    }
+}
+
 static void
 accounts_dialog_button_add_clicked_cb (GtkWidget *button,
     EmpathyAccountsDialog *dialog)
 {
-  EmpathyAccount *account = NULL;
+  TpAccount *account = NULL;
 
   if (accounts_dialog_has_pending_change (dialog, &account))
     {
-      gchar *question_dialog_primary_text = g_strdup_printf (
-          PENDING_CHANGES_QUESTION_PRIMARY_TEXT,
-          empathy_account_get_display_name (account));
+      gchar *question_dialog_primary_text = get_dialog_primary_text (account);
 
       accounts_dialog_show_question_dialog (dialog,
           question_dialog_primary_text,
@@ -545,14 +514,19 @@ accounts_dialog_update_settings (EmpathyAccountsDialog *dialog,
     {
       GtkTreeView  *view;
       GtkTreeModel *model;
+      GtkTreeSelection *selection;
 
       view = GTK_TREE_VIEW (priv->treeview);
       model = gtk_tree_view_get_model (view);
+      selection = gtk_tree_view_get_selection (view);
 
       if (gtk_tree_model_iter_n_children (model, NULL) > 0)
         {
-          /* We have configured accounts, select the first one */
-          accounts_dialog_model_select_first (dialog);
+          /* We have configured accounts, select the first one if there
+           * is no other account selected already. */
+          if (!gtk_tree_selection_get_selected (selection, NULL, NULL))
+            accounts_dialog_model_select_first (dialog);
+
           return;
         }
       if (empathy_connection_managers_get_cms_num (priv->cms) > 0)
@@ -708,9 +682,11 @@ accounts_dialog_name_edited_cb (GtkCellRendererText *renderer,
   GtkTreePath  *treepath;
   GtkTreeIter   iter;
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  gboolean connecting;
 
-  if (empathy_account_manager_get_connecting_accounts
-      (priv->account_manager) > 0)
+  empathy_account_manager_get_accounts_connected (&connecting);
+
+  if (connecting)
     {
       priv->connecting_id = g_timeout_add (FLASH_TIMEOUT,
           (GSourceFunc) accounts_dialog_flash_connecting_cb,
@@ -739,7 +715,7 @@ accounts_dialog_delete_account_response_cb (GtkDialog *message_dialog,
   gint response_id,
   gpointer user_data)
 {
-  EmpathyAccount *account;
+  TpAccount *account;
   GtkTreeModel *model;
   GtkTreeIter iter;
   GtkTreeSelection *selection;
@@ -757,7 +733,7 @@ accounts_dialog_delete_account_response_cb (GtkDialog *message_dialog,
 
       if (account != NULL)
         {
-          empathy_account_remove_async (account, NULL, NULL);
+          tp_account_remove_async (account, NULL, NULL);
           g_object_unref (account);
           account = NULL;
         }
@@ -774,7 +750,7 @@ accounts_dialog_view_delete_activated_cb (EmpathyCellRendererActivatable *cell,
     const gchar *path_string,
     EmpathyAccountsDialog *dialog)
 {
-  EmpathyAccount *account;
+  TpAccount *account;
   GtkTreeModel *model;
   GtkTreeIter iter;
   gchar *question_dialog_primary_text;
@@ -787,7 +763,7 @@ accounts_dialog_view_delete_activated_cb (EmpathyCellRendererActivatable *cell,
 
   gtk_tree_model_get (model, &iter, COL_ACCOUNT_POINTER, &account, -1);
 
-  if (account == NULL || !empathy_account_is_valid (account))
+  if (account == NULL || !tp_account_is_valid (account))
     {
       gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
       accounts_dialog_model_select_first (dialog);
@@ -795,26 +771,11 @@ accounts_dialog_view_delete_activated_cb (EmpathyCellRendererActivatable *cell,
     }
 
   question_dialog_primary_text = g_strdup_printf (
-#ifndef HAVE_MOBLIN
-      _("You are about to remove your %s account!\n"
-          "Are you sure you want to proceed?"),
-#else
-      /* Translators: this is used only when built on a moblin platform */
       _("Do you want to remove %s from your computer?"),
-#endif /* HAVE_MOBLIN */
-      empathy_account_get_display_name (account));
+      tp_account_get_display_name (account));
 
   accounts_dialog_show_question_dialog (dialog, question_dialog_primary_text,
-#ifndef HAVE_MOBLIN
-      _("Any associated conversations and chat rooms will NOT be "
-          "removed if you decide to proceed.\n"
-          "\n"
-          "Should you decide to add the account back at a later time, "
-          "they will still be available."),
-#else
-      /* Translators: this is used only when built on a moblin platform */
       _("This will not remove your account on the server."),
-#endif /* HAVE_MOBLIN */
       G_CALLBACK (accounts_dialog_delete_account_response_cb),
       dialog,
       GTK_STOCK_CANCEL, GTK_RESPONSE_NO,
@@ -853,9 +814,6 @@ accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
       accounts_dialog_model_pixbuf_data_func,
       dialog,
       NULL);
-#ifdef HAVE_MOBLIN
-  g_object_set (cell, "ypad", 4, NULL);
-#endif
 
   /* Name renderer */
   cell = gtk_cell_renderer_text_new ();
@@ -872,15 +830,14 @@ accounts_dialog_model_add_columns (EmpathyAccountsDialog *dialog)
   g_signal_connect (cell, "editing-started",
       G_CALLBACK (accounts_dialog_name_editing_started_cb),
       dialog);
+  g_object_set (cell, "ypad", 4, NULL);
 
   /* Delete column */
   cell = empathy_cell_renderer_activatable_new ();
   gtk_tree_view_column_pack_start (column, cell, FALSE);
   g_object_set (cell,
         "icon-name", GTK_STOCK_DELETE,
-#ifdef HAVE_MOBLIN
         "show-on-select", TRUE,
-#endif
         NULL);
 
   g_signal_connect (cell, "path-activated",
@@ -977,7 +934,7 @@ accounts_dialog_account_selection_change (GtkTreeSelection *selection,
     gboolean path_currently_selected,
     gpointer data)
 {
-  EmpathyAccount *account = NULL;
+  TpAccount *account = NULL;
   EmpathyAccountsDialog *dialog = EMPATHY_ACCOUNTS_DIALOG (data);
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
@@ -995,13 +952,9 @@ accounts_dialog_account_selection_change (GtkTreeSelection *selection,
       /* The currently selected account has some unsaved changes. We ask
        * the user if he really wants to lose his changes and select another
        * account */
-      gchar *question_dialog_primary_text;
+      gchar *question_dialog_primary_text = get_dialog_primary_text (account);
       priv->destination_row = gtk_tree_row_reference_new (model, path);
 
-      question_dialog_primary_text = g_strdup_printf (
-          PENDING_CHANGES_QUESTION_PRIMARY_TEXT,
-          empathy_account_get_display_name (account));
-
       accounts_dialog_show_question_dialog (dialog,
           question_dialog_primary_text,
           _("You are about to select another account, which will discard\n"
@@ -1031,7 +984,7 @@ accounts_dialog_model_setup (EmpathyAccountsDialog *dialog)
   store = gtk_list_store_new (COL_COUNT,
       G_TYPE_STRING,         /* name */
       G_TYPE_UINT,           /* status */
-      EMPATHY_TYPE_ACCOUNT,   /* account */
+      TP_TYPE_ACCOUNT,   /* account */
       EMPATHY_TYPE_ACCOUNT_SETTINGS); /* settings */
 
   gtk_tree_view_set_model (GTK_TREE_VIEW (priv->treeview),
@@ -1093,7 +1046,7 @@ accounts_dialog_get_settings_iter (EmpathyAccountsDialog *dialog,
 
 static gboolean
 accounts_dialog_get_account_iter (EmpathyAccountsDialog *dialog,
-    EmpathyAccount *account,
+    TpAccount *account,
     GtkTreeIter *iter)
 {
   GtkTreeView      *view;
@@ -1162,11 +1115,12 @@ accounts_dialog_add (EmpathyAccountsDialog *dialog,
 }
 
 static void
-accounts_dialog_connection_changed_cb     (EmpathyAccountManager *manager,
-    EmpathyAccount *account,
-    TpConnectionStatusReason reason,
-    TpConnectionStatus current,
-    TpConnectionStatus previous,
+accounts_dialog_connection_changed_cb (TpAccount *account,
+    guint old_status,
+    guint current,
+    guint reason,
+    gchar *dbus_error_name,
+    GHashTable *details,
     EmpathyAccountsDialog *dialog)
 {
   GtkTreeModel *model;
@@ -1190,7 +1144,7 @@ accounts_dialog_connection_changed_cb     (EmpathyAccountManager *manager,
       gtk_tree_path_free (path);
     }
 
-  found = (empathy_account_manager_get_connecting_accounts (manager) > 0);
+  empathy_account_manager_get_accounts_connected (&found);
 
   if (!found && priv->connecting_id)
     {
@@ -1205,7 +1159,7 @@ accounts_dialog_connection_changed_cb     (EmpathyAccountManager *manager,
 }
 
 static void
-accounts_dialog_account_display_name_changed_cb (EmpathyAccount *account,
+accounts_dialog_account_display_name_changed_cb (TpAccount *account,
   GParamSpec *pspec,
   gpointer user_data)
 {
@@ -1213,11 +1167,11 @@ accounts_dialog_account_display_name_changed_cb (EmpathyAccount *account,
   GtkTreeIter iter;
   GtkTreeModel *model;
   EmpathyAccountSettings *settings;
-  EmpathyAccount *selected_account;
+  TpAccount *selected_account;
   EmpathyAccountsDialog *dialog = EMPATHY_ACCOUNTS_DIALOG (user_data);
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
-  display_name = empathy_account_get_display_name (account);
+  display_name = tp_account_get_display_name (account);
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
   settings = accounts_dialog_model_get_selected_settings (dialog);
   if (settings == NULL)
@@ -1240,7 +1194,7 @@ accounts_dialog_account_display_name_changed_cb (EmpathyAccount *account,
 
 static void
 accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
-    EmpathyAccount *account)
+    TpAccount *account)
 {
   EmpathyAccountSettings *settings;
   GtkTreeModel       *model;
@@ -1251,9 +1205,9 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
 
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
-  g_object_get (account, "connection-status", &status, NULL);
-  name = empathy_account_get_display_name (account);
-  enabled = empathy_account_is_enabled (account);
+  status = tp_account_get_connection_status (account, NULL);
+  name = tp_account_get_display_name (account);
+  enabled = tp_account_is_enabled (account);
 
   settings = empathy_account_settings_new_for_account (account);
 
@@ -1267,11 +1221,12 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
       COL_ACCOUNT_SETTINGS_POINTER, settings,
       -1);
 
-  accounts_dialog_connection_changed_cb (priv->account_manager,
-      account,
-      TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED,
+  accounts_dialog_connection_changed_cb (account,
+      0,
       status,
-      TP_CONNECTION_STATUS_DISCONNECTED,
+      TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED,
+      NULL,
+      NULL,
       dialog);
 
   empathy_signal_connect_weak (account, "notify::display-name",
@@ -1282,48 +1237,36 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
 }
 
 static void
-accounts_dialog_update (EmpathyAccountsDialog *dialog,
-    EmpathyAccountSettings *settings)
+account_prepare_cb (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data)
 {
-  GtkTreeModel       *model;
-  GtkTreeIter         iter;
-  TpConnectionStatus  status = TP_CONNECTION_STATUS_DISCONNECTED;
-  const gchar        *name;
-  gboolean            enabled = FALSE;
-  EmpathyAccount     *account;
-  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
+  EmpathyAccountsDialog *dialog = EMPATHY_ACCOUNTS_DIALOG (user_data);
+  TpAccount *account = TP_ACCOUNT (source_object);
+  GError *error = NULL;
 
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
-  name = empathy_account_settings_get_display_name (settings);
-
-  account = empathy_account_settings_get_account (settings);
-  if (account != NULL)
+  if (!tp_account_prepare_finish (account, result, &error))
     {
-      enabled = empathy_account_is_enabled (account);
-      g_object_get (account, "connection-status", &status, NULL);
+      DEBUG ("Failed to prepare account: %s", error->message);
+      g_error_free (error);
+      return;
     }
 
-  accounts_dialog_get_settings_iter (dialog, settings, &iter);
-  gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-      COL_NAME, name,
-      COL_STATUS, status,
-      COL_ACCOUNT_POINTER, account,
-      COL_ACCOUNT_SETTINGS_POINTER, settings,
-      -1);
-}
-
-static void
-accounts_dialog_account_added_cb (EmpathyAccountManager *manager,
-    EmpathyAccount *account,
-    EmpathyAccountsDialog *dialog)
-{
   accounts_dialog_add_account (dialog, account);
 }
 
+static void
+accounts_dialog_account_validity_changed_cb (TpAccountManager *manager,
+    TpAccount *account,
+    gboolean valid,
+    EmpathyAccountsDialog *dialog)
+{
+  tp_account_prepare_async (account, NULL, account_prepare_cb, dialog);
+}
 
 static void
-accounts_dialog_account_removed_cb (EmpathyAccountManager *manager,
-    EmpathyAccount *account,
+accounts_dialog_account_removed_cb (TpAccountManager *manager,
+    TpAccount *account,
     EmpathyAccountsDialog *dialog)
 {
   GtkTreeIter iter;
@@ -1338,7 +1281,7 @@ accounts_dialog_account_removed_cb (EmpathyAccountManager *manager,
 
 static void
 enable_or_disable_account (EmpathyAccountsDialog *dialog,
-    EmpathyAccount *account,
+    TpAccount *account,
     gboolean enabled)
 {
   GtkTreeModel *model;
@@ -1348,59 +1291,26 @@ enable_or_disable_account (EmpathyAccountsDialog *dialog,
   model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
 
   DEBUG ("Account %s is now %s",
-      empathy_account_get_display_name (account),
+      tp_account_get_display_name (account),
       enabled ? "enabled" : "disabled");
 }
 
 static void
-accounts_dialog_account_disabled_cb (EmpathyAccountManager *manager,
-    EmpathyAccount *account,
+accounts_dialog_account_disabled_cb (TpAccountManager *manager,
+    TpAccount *account,
     EmpathyAccountsDialog *dialog)
 {
   enable_or_disable_account (dialog, account, FALSE);
 }
 
 static void
-accounts_dialog_account_enabled_cb (EmpathyAccountManager *manager,
-    EmpathyAccount *account,
+accounts_dialog_account_enabled_cb (TpAccountManager *manager,
+    TpAccount *account,
     EmpathyAccountsDialog *dialog)
 {
   enable_or_disable_account (dialog, account, TRUE);
 }
 
-static void
-accounts_dialog_account_changed_cb (EmpathyAccountManager *manager,
-    EmpathyAccount *account,
-    EmpathyAccountsDialog  *dialog)
-{
-  EmpathyAccountSettings *settings, *selected_settings;
-  GtkTreeModel *model;
-  GtkTreeIter iter;
-  EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
-
-  model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
-
-  if (!accounts_dialog_get_account_iter (dialog, account, &iter))
-    return;
-
-  gtk_tree_model_get (model, &iter,
-      COL_ACCOUNT_SETTINGS_POINTER, &settings,
-      -1);
-
-  accounts_dialog_update (dialog, settings);
-  selected_settings = accounts_dialog_model_get_selected_settings (dialog);
-
-  if (settings == selected_settings)
-    accounts_dialog_update_name_label (dialog,
-        empathy_account_settings_get_display_name (settings));
-
-  if (settings)
-    g_object_unref (settings);
-
-  if (selected_settings)
-    g_object_unref (selected_settings);
-}
-
 static void
 accounts_dialog_button_create_clicked_cb (GtkWidget *button,
     EmpathyAccountsDialog *dialog)
@@ -1504,14 +1414,11 @@ accounts_dialog_response_cb (GtkWidget *widget,
     gint response,
     EmpathyAccountsDialog *dialog)
 {
-  EmpathyAccount *account = NULL;
+  TpAccount *account = NULL;
 
   if (accounts_dialog_has_pending_change (dialog, &account))
     {
-      gchar *question_dialog_primary_text;
-      question_dialog_primary_text = g_strdup_printf (
-          PENDING_CHANGES_QUESTION_PRIMARY_TEXT,
-          empathy_account_get_display_name (account));
+      gchar *question_dialog_primary_text = get_dialog_primary_text (account);
 
       accounts_dialog_show_question_dialog (dialog,
           question_dialog_primary_text,
@@ -1549,7 +1456,7 @@ accounts_dialog_destroy_cb (GtkObject *obj,
 
 static void
 accounts_dialog_set_selected_account (EmpathyAccountsDialog *dialog,
-    EmpathyAccount *account)
+    TpAccount *account)
 {
   GtkTreeSelection *selection;
   GtkTreeIter       iter;
@@ -1587,10 +1494,10 @@ accounts_dialog_accounts_setup (EmpathyAccountsDialog *dialog)
   EmpathyAccountsDialogPriv *priv = GET_PRIV (dialog);
   GList *accounts, *l;
 
-  g_signal_connect (priv->account_manager, "account-created",
-      G_CALLBACK (accounts_dialog_account_added_cb),
+  g_signal_connect (priv->account_manager, "account-validity-changed",
+      G_CALLBACK (accounts_dialog_account_validity_changed_cb),
       dialog);
-  g_signal_connect (priv->account_manager, "account-deleted",
+  g_signal_connect (priv->account_manager, "account-removed",
       G_CALLBACK (accounts_dialog_account_removed_cb),
       dialog);
   g_signal_connect (priv->account_manager, "account-enabled",
@@ -1599,19 +1506,15 @@ accounts_dialog_accounts_setup (EmpathyAccountsDialog *dialog)
   g_signal_connect (priv->account_manager, "account-disabled",
       G_CALLBACK (accounts_dialog_account_disabled_cb),
       dialog);
-  g_signal_connect (priv->account_manager, "account-changed",
-      G_CALLBACK (accounts_dialog_account_changed_cb),
-      dialog);
-  g_signal_connect (priv->account_manager, "account-connection-changed",
-      G_CALLBACK (accounts_dialog_connection_changed_cb),
-      dialog);
 
   /* Add existing accounts */
-  accounts = empathy_account_manager_dup_accounts (priv->account_manager);
+  accounts = tp_account_manager_get_valid_accounts (priv->account_manager);
   for (l = accounts; l; l = l->next)
     {
       accounts_dialog_add_account (dialog, l->data);
-      g_object_unref (l->data);
+
+      empathy_signal_connect_weak (l->data, "status-changed",
+          G_CALLBACK (accounts_dialog_connection_changed_cb), G_OBJECT (dialog));
     }
   g_list_free (accounts);
 
@@ -1624,12 +1527,19 @@ accounts_dialog_accounts_setup (EmpathyAccountsDialog *dialog)
 }
 
 static void
-accounts_dialog_manager_ready_cb (EmpathyAccountManager *manager,
-    GParamSpec *pspec,
+accounts_dialog_manager_ready_cb (GObject *source_object,
+    GAsyncResult *result,
     gpointer user_data)
 {
-  if (!empathy_account_manager_is_ready (manager))
-    return;
+  TpAccountManager *manager = TP_ACCOUNT_MANAGER (source_object);
+  GError *error = NULL;
+
+  if (!tp_account_manager_prepare_finish (manager, result, &error))
+    {
+      DEBUG ("Failed to prepare account manager: %s", error->message);
+      g_error_free (error);
+      return;
+    }
 
   accounts_dialog_accounts_setup (user_data);
 }
@@ -1709,7 +1619,7 @@ do_dispose (GObject *obj)
 
   /* Disconnect signals */
   g_signal_handlers_disconnect_by_func (priv->account_manager,
-      accounts_dialog_account_added_cb,
+      accounts_dialog_account_validity_changed_cb,
       dialog);
   g_signal_handlers_disconnect_by_func (priv->account_manager,
       accounts_dialog_account_removed_cb,
@@ -1720,12 +1630,6 @@ do_dispose (GObject *obj)
   g_signal_handlers_disconnect_by_func (priv->account_manager,
       accounts_dialog_account_disabled_cb,
       dialog);
-  g_signal_handlers_disconnect_by_func (priv->account_manager,
-      accounts_dialog_account_changed_cb,
-      dialog);
-  g_signal_handlers_disconnect_by_func (priv->account_manager,
-      accounts_dialog_connection_changed_cb,
-      dialog);
   g_signal_handlers_disconnect_by_func (priv->account_manager,
       accounts_dialog_manager_ready_cb,
       dialog);
@@ -1772,6 +1676,9 @@ do_constructor (GType type,
 
       dialog_singleton = EMPATHY_ACCOUNTS_DIALOG (retval);
       g_object_add_weak_pointer (retval, (gpointer) &dialog_singleton);
+      /* We add an extra reference that we'll release when the dialog is
+       * destroyed (accounts_dialog_destroy_cb) */
+      g_object_ref (retval);
     }
 
   return retval;
@@ -1824,13 +1731,10 @@ do_constructed (GObject *object)
   accounts_dialog_model_setup (dialog);
 
   /* Set up signalling */
-  priv->account_manager = empathy_account_manager_dup_singleton ();
+  priv->account_manager = tp_account_manager_dup ();
 
-  if (!empathy_account_manager_is_ready (priv->account_manager))
-    g_signal_connect (priv->account_manager, "notify::ready",
-        G_CALLBACK (accounts_dialog_manager_ready_cb), dialog);
-  else
-    accounts_dialog_accounts_setup (dialog);
+  tp_account_manager_prepare_async (priv->account_manager, NULL,
+      accounts_dialog_manager_ready_cb, dialog);
 
   empathy_conf_get_bool (empathy_conf_get (),
       EMPATHY_PREFS_IMPORT_ASKED, &import_asked);
@@ -1888,7 +1792,7 @@ empathy_accounts_dialog_init (EmpathyAccountsDialog *dialog)
 
 GtkWidget *
 empathy_accounts_dialog_show (GtkWindow *parent,
-    EmpathyAccount *selected_account)
+    TpAccount *selected_account)
 {
   EmpathyAccountsDialog *dialog;
   EmpathyAccountsDialogPriv *priv;
@@ -1900,7 +1804,7 @@ empathy_accounts_dialog_show (GtkWindow *parent,
 
   if (selected_account)
     {
-      if (empathy_connection_managers_is_ready (priv->cms))
+      if (priv->cms != NULL && empathy_connection_managers_is_ready (priv->cms))
         accounts_dialog_set_selected_account (dialog, selected_account);
       else
         /* save the selection to set it later when the cms
@@ -1910,6 +1814,9 @@ empathy_accounts_dialog_show (GtkWindow *parent,
     }
 
   gtk_window_present (GTK_WINDOW (priv->window));
+  /* EmpathyAccountsDialog kepts a ref on itself until the dialog is
+   * destroyed so we can release the ref returned by the constructor now. */
+  g_object_unref (dialog);
 
   return priv->window;
 }
