@@ -27,9 +27,11 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
+#include <telepathy-glib/account-manager.h>
+
 #include <libempathy/empathy-contact.h>
+#include <libempathy/empathy-idle.h>
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-account-manager.h>
 #include <libempathy/empathy-dispatcher.h>
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-chatroom.h>
@@ -79,7 +81,7 @@
 typedef struct {
 	EmpathyContactListView  *list_view;
 	EmpathyContactListStore *list_store;
-	EmpathyAccountManager   *account_manager;
+	TpAccountManager        *account_manager;
 	EmpathyChatroomManager  *chatroom_manager;
 	EmpathyEventManager     *event_manager;
 	guint                    flash_timeout_id;
@@ -113,7 +115,7 @@ typedef struct {
 	GList                  *actions_connected;
 } EmpathyMainWindow;
 
-static EmpathyMainWindow *window = NULL;
+static EmpathyMainWindow *main_window = NULL;
 
 static void
 main_window_flash_stop (EmpathyMainWindow *window)
@@ -302,10 +304,25 @@ main_window_row_activated_cb (EmpathyContactListView *view,
 }
 
 static void
-main_window_error_edit_clicked_cb (GtkButton         *button,
+main_window_error_retry_clicked_cb (GtkButton *button,
 				   EmpathyMainWindow *window)
 {
-	EmpathyAccount *account;
+	TpAccount *account;
+	GtkWidget *error_widget;
+
+	account = g_object_get_data (G_OBJECT (button), "account");
+	tp_account_reconnect_async (account, NULL, NULL);
+
+	error_widget = g_hash_table_lookup (window->errors, account);
+	gtk_widget_destroy (error_widget);
+	g_hash_table_remove (window->errors, account);
+}
+
+static void
+main_window_error_edit_clicked_cb (GtkButton *button,
+				   EmpathyMainWindow *window)
+{
+	TpAccount *account;
 	GtkWidget *error_widget;
 
 	account = g_object_get_data (G_OBJECT (button), "account");
@@ -317,10 +334,10 @@ main_window_error_edit_clicked_cb (GtkButton         *button,
 }
 
 static void
-main_window_error_clear_clicked_cb (GtkButton         *button,
+main_window_error_close_clicked_cb (GtkButton *button,
 				    EmpathyMainWindow *window)
 {
-	EmpathyAccount *account;
+	TpAccount *account;
 	GtkWidget *error_widget;
 
 	account = g_object_get_data (G_OBJECT (button), "account");
@@ -331,147 +348,139 @@ main_window_error_clear_clicked_cb (GtkButton         *button,
 
 static void
 main_window_error_display (EmpathyMainWindow *window,
-			   EmpathyAccount    *account,
+			   TpAccount         *account,
 			   const gchar       *message)
 {
-	GtkWidget *child;
-	GtkWidget *table;
-	GtkWidget *image;
-	GtkWidget *button_edit;
-	GtkWidget *alignment;
-	GtkWidget *hbox;
+	GtkWidget *info_bar;
+	GtkWidget *content_area;
 	GtkWidget *label;
-	GtkWidget *fixed;
-	GtkWidget *vbox;
-	GtkWidget *button_close;
+	GtkWidget *image;
+	GtkWidget *retry_button;
+	GtkWidget *edit_button;
+	GtkWidget *close_button;
+	GtkWidget *action_area;
+	GtkWidget *action_table;
+	GtkRcStyle *rc_style;
 	gchar     *str;
+	const gchar     *icon_name;
 
-	child = g_hash_table_lookup (window->errors, account);
-	if (child) {
-		label = g_object_get_data (G_OBJECT (child), "label");
+	str = g_markup_printf_escaped ("<b>%s</b>\n%s",
+					       tp_account_get_display_name (account),
+					       message);
+
+	info_bar = g_hash_table_lookup (window->errors, account);
+	if (info_bar) {
+		label = g_object_get_data (G_OBJECT (info_bar), "label");
 
 		/* Just set the latest error and return */
-		str = g_markup_printf_escaped ("<b>%s</b>\n%s",
-					       empathy_account_get_display_name (account),
-					       message);
 		gtk_label_set_markup (GTK_LABEL (label), str);
 		g_free (str);
 
 		return;
 	}
 
-	child = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (window->errors_vbox), child, FALSE, TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (child), 6);
-	gtk_widget_show (child);
+	info_bar = gtk_info_bar_new ();
+	gtk_info_bar_set_message_type (GTK_INFO_BAR (info_bar), GTK_MESSAGE_WARNING);
 
-	table = gtk_table_new (2, 4, FALSE);
-	gtk_widget_show (table);
-	gtk_box_pack_start (GTK_BOX (child), table, TRUE, TRUE, 0);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 12);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+	gtk_widget_set_no_show_all (info_bar, TRUE);
+	gtk_box_pack_start (GTK_BOX (window->errors_vbox), info_bar, FALSE, TRUE, 0);
+	gtk_widget_show (info_bar);
 
-	image = gtk_image_new_from_stock (GTK_STOCK_DISCONNECT, GTK_ICON_SIZE_MENU);
+	icon_name = tp_account_get_icon_name (account);
+	image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_SMALL_TOOLBAR);
 	gtk_widget_show (image);
-	gtk_table_attach (GTK_TABLE (table), image, 0, 1, 0, 2,
-			  (GtkAttachOptions) (GTK_FILL),
-			  (GtkAttachOptions) (GTK_FILL), 0, 0);
-	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0);
 
-	button_edit = gtk_button_new ();
-	gtk_widget_show (button_edit);
-	gtk_table_attach (GTK_TABLE (table), button_edit, 1, 2, 1, 2,
-			  (GtkAttachOptions) (GTK_FILL),
-			  (GtkAttachOptions) (0), 0, 0);
-
-	alignment = gtk_alignment_new (0.5, 0.5, 0, 0);
-	gtk_widget_show (alignment);
-	gtk_container_add (GTK_CONTAINER (button_edit), alignment);
-
-	hbox = gtk_hbox_new (FALSE, 2);
-	gtk_widget_show (hbox);
-	gtk_container_add (GTK_CONTAINER (alignment), hbox);
-
-	image = gtk_image_new_from_stock (GTK_STOCK_EDIT, GTK_ICON_SIZE_BUTTON);
-	gtk_widget_show (image);
-	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-
-	label = gtk_label_new_with_mnemonic (_("_Edit account"));
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-
-	fixed = gtk_fixed_new ();
-	gtk_widget_show (fixed);
-	gtk_table_attach (GTK_TABLE (table), fixed, 2, 3, 1, 2,
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions) (GTK_FILL), 0, 0);
-
-	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_widget_show (vbox);
-	gtk_table_attach (GTK_TABLE (table), vbox, 3, 4, 0, 2,
-			  (GtkAttachOptions) (GTK_FILL),
-			  (GtkAttachOptions) (GTK_FILL), 0, 0);
-
-	button_close = gtk_button_new ();
-	gtk_widget_show (button_close);
-	gtk_box_pack_start (GTK_BOX (vbox), button_close, FALSE, FALSE, 0);
-	gtk_button_set_relief (GTK_BUTTON (button_close), GTK_RELIEF_NONE);
-
-
-	image = gtk_image_new_from_stock ("gtk-close", GTK_ICON_SIZE_MENU);
-	gtk_widget_show (image);
-	gtk_container_add (GTK_CONTAINER (button_close), image);
-
-	label = gtk_label_new ("");
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 1, 3, 0, 1,
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL), 0, 0);
-	gtk_widget_set_size_request (label, 175, -1);
+	label = gtk_label_new (str);
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
-
-	str = g_markup_printf_escaped ("<b>%s</b>\n%s",
-				       empathy_account_get_display_name (account),
-				       message);
-	gtk_label_set_markup (GTK_LABEL (label), str);
+	gtk_widget_show (label);
 	g_free (str);
 
-	g_object_set_data (G_OBJECT (child), "label", label);
-	g_object_set_data_full (G_OBJECT (button_edit),
+	content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (info_bar));
+	gtk_box_pack_start (GTK_BOX (content_area), image, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (content_area), label, FALSE, FALSE, 0);
+
+	/* make small style for the buttons */
+	rc_style = gtk_rc_style_new ();
+	rc_style->xthickness = rc_style->ythickness = 4;
+
+	image = gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_BUTTON);
+	retry_button = gtk_button_new ();
+	gtk_button_set_image (GTK_BUTTON (retry_button), image);
+	gtk_widget_modify_style (retry_button, rc_style);
+	gtk_widget_set_tooltip_text (retry_button, _("Reconnect"));
+	gtk_widget_show (retry_button);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_EDIT, GTK_ICON_SIZE_BUTTON);
+	edit_button = gtk_button_new ();
+	gtk_button_set_image (GTK_BUTTON (edit_button), image);
+	gtk_widget_modify_style (edit_button, rc_style);
+	gtk_widget_set_tooltip_text (edit_button, _("Edit Account"));
+	gtk_widget_show (edit_button);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_BUTTON);
+	close_button = gtk_button_new ();
+	gtk_button_set_image (GTK_BUTTON (close_button), image);
+	gtk_widget_modify_style (close_button, rc_style);
+	gtk_widget_set_tooltip_text (close_button, _("Close"));
+	gtk_widget_show (close_button);
+
+	action_table = gtk_table_new (1, 3, FALSE);
+	gtk_table_set_col_spacings (GTK_TABLE (action_table), 2);
+	gtk_widget_show (action_table);
+
+	action_area = gtk_info_bar_get_action_area (GTK_INFO_BAR (info_bar));
+	gtk_box_pack_start (GTK_BOX (action_area), action_table, FALSE, FALSE, 0);
+
+	gtk_table_attach (GTK_TABLE (action_table), retry_button, 0, 1, 0, 1,
+										(GtkAttachOptions) (GTK_SHRINK),
+										(GtkAttachOptions) (GTK_SHRINK), 0, 0);
+	gtk_table_attach (GTK_TABLE (action_table), edit_button, 1, 2, 0, 1,
+										(GtkAttachOptions) (GTK_SHRINK),
+										(GtkAttachOptions) (GTK_SHRINK), 0, 0);
+	gtk_table_attach (GTK_TABLE (action_table), close_button, 2, 3, 0, 1,
+										(GtkAttachOptions) (GTK_SHRINK),
+										(GtkAttachOptions) (GTK_SHRINK), 0, 0);
+
+	g_object_set_data (G_OBJECT (info_bar), "label", label);
+	g_object_set_data_full (G_OBJECT (info_bar),
 				"account", g_object_ref (account),
 				g_object_unref);
-	g_object_set_data_full (G_OBJECT (button_close),
+	g_object_set_data_full (G_OBJECT (edit_button),
+				"account", g_object_ref (account),
+				g_object_unref);
+	g_object_set_data_full (G_OBJECT (close_button),
+				"account", g_object_ref (account),
+				g_object_unref);
+	g_object_set_data_full (G_OBJECT (retry_button),
 				"account", g_object_ref (account),
 				g_object_unref);
 
-	g_signal_connect (button_edit, "clicked",
+	g_signal_connect (edit_button, "clicked",
 			  G_CALLBACK (main_window_error_edit_clicked_cb),
 			  window);
-
-	g_signal_connect (button_close, "clicked",
-			  G_CALLBACK (main_window_error_clear_clicked_cb),
+	g_signal_connect (close_button, "clicked",
+			  G_CALLBACK (main_window_error_close_clicked_cb),
+			  window);
+	g_signal_connect (retry_button, "clicked",
+			  G_CALLBACK (main_window_error_retry_clicked_cb),
 			  window);
 
 	gtk_widget_show (window->errors_vbox);
 
-	g_hash_table_insert (window->errors, g_object_ref (account), child);
+	g_object_unref (rc_style);
+	g_hash_table_insert (window->errors, g_object_ref (account), info_bar);
 }
 
 static void
-main_window_update_status (EmpathyMainWindow *window, EmpathyAccountManager *manager)
+main_window_update_status (EmpathyMainWindow *window)
 {
-	int  connected;
-	int  connecting;
+	gboolean connected, connecting;
 	GList *l;
 
-	/* Count number of connected/connecting/disconnected accounts */
-	connected = empathy_account_manager_get_connected_accounts (manager);
-	connecting = empathy_account_manager_get_connecting_accounts (manager);
+	connected = empathy_account_manager_get_accounts_connected (&connecting);
 
 	/* Update the spinner state */
-	if (connecting > 0) {
+	if (connecting) {
 		ephy_spinner_start (EPHY_SPINNER (window->throbber));
 	} else {
 		ephy_spinner_stop (EPHY_SPINNER (window->throbber));
@@ -479,19 +488,20 @@ main_window_update_status (EmpathyMainWindow *window, EmpathyAccountManager *man
 
 	/* Update widgets sensibility */
 	for (l = window->actions_connected; l; l = l->next) {
-		gtk_action_set_sensitive (l->data, (connected > 0));
+		gtk_action_set_sensitive (l->data, connected);
 	}
 }
 
 static void
-main_window_connection_changed_cb (EmpathyAccountManager *manager,
-				   EmpathyAccount *account,
-				   TpConnectionStatusReason reason,
-				   TpConnectionStatus current,
-				   TpConnectionStatus previous,
+main_window_connection_changed_cb (TpAccount  *account,
+                                   guint       old_status,
+                                   guint       current,
+                                   guint       reason,
+                                   gchar      *dbus_error_name,
+                                   GHashTable *details,
 				   EmpathyMainWindow *window)
 {
-	main_window_update_status (window, manager);
+	main_window_update_status (window);
 
 	if (current == TP_CONNECTION_STATUS_DISCONNECTED &&
 	    reason != TP_CONNECTION_STATUS_REASON_REQUESTED) {
@@ -572,15 +582,17 @@ main_window_contact_presence_changed_cb (EmpathyContactMonitor *monitor,
 					 TpConnectionPresenceType previous,
 					 EmpathyMainWindow *window)
 {
-	EmpathyAccount *account;
-	gboolean should_play;
+  TpAccount *account;
+  gboolean should_play = FALSE;
+  EmpathyIdle *idle;
 
-	account = empathy_contact_get_account (contact);
-	should_play = !empathy_account_is_just_connected (account);
+  account = empathy_contact_get_account (contact);
+  idle = empathy_idle_dup_singleton ();
 
-	if (!should_play) {
-		return;
-	}
+  should_play = !empathy_idle_account_is_just_connected (idle, account);
+
+  if (!should_play)
+    goto out;
 
   if (tp_connection_presence_type_cmp_availability (previous,
      TP_CONNECTION_PRESENCE_TYPE_OFFLINE) > 0)
@@ -601,6 +613,9 @@ main_window_contact_presence_changed_cb (EmpathyContactMonitor *monitor,
         empathy_sound_play (GTK_WIDGET (window->window),
           EMPATHY_SOUND_CONTACT_CONNECTED);
     }
+
+out:
+  g_object_unref (idle);
 }
 
 static void
@@ -641,14 +656,6 @@ main_window_destroy_cb (GtkWidget         *widget,
 	/* Save user-defined accelerators. */
 	main_window_accels_save ();
 
-	g_signal_handlers_disconnect_by_func (window->account_manager,
-					      main_window_connection_changed_cb,
-					      window);
-
-	if (window->size_timeout_id) {
-		g_source_remove (window->size_timeout_id);
-	}
-
 	g_list_free (window->actions_connected);
 
 	g_object_unref (window->account_manager);
@@ -845,12 +852,12 @@ main_window_view_show_map_cb (GtkCheckMenuItem  *item,
 static void
 main_window_favorite_chatroom_join (EmpathyChatroom *chatroom)
 {
-	EmpathyAccount *account;
+	TpAccount      *account;
 	TpConnection   *connection;
 	const gchar    *room;
 
 	account = empathy_chatroom_get_account (chatroom);
-	connection = empathy_account_get_connection (account);
+	connection = tp_account_get_connection (account);
 	room = empathy_chatroom_get_room (chatroom);
 
 	if (connection != NULL) {
@@ -1064,7 +1071,7 @@ static void
 main_window_help_debug_cb (GtkAction         *action,
 			   EmpathyMainWindow *window)
 {
-	empathy_debug_window_new (GTK_WINDOW (window->window));
+	empathy_debug_window_new (NULL);
 }
 
 static void
@@ -1089,44 +1096,28 @@ main_window_throbber_button_press_event_cb (GtkWidget         *throbber_ebox,
 	return FALSE;
 }
 
-static gboolean
-main_window_configure_event_timeout_cb (EmpathyMainWindow *window)
-{
-	gint x, y, w, h;
-
-	gtk_window_get_size (GTK_WINDOW (window->window), &w, &h);
-	gtk_window_get_position (GTK_WINDOW (window->window), &x, &y);
-
-	empathy_geometry_save (GEOMETRY_NAME, x, y, w, h);
-
-	window->size_timeout_id = 0;
-
-	return FALSE;
-}
-
-static gboolean
-main_window_configure_event_cb (GtkWidget         *widget,
-				GdkEventConfigure *event,
+static void
+main_window_account_removed_cb (TpAccountManager  *manager,
+				TpAccount         *account,
 				EmpathyMainWindow *window)
 {
-	if (window->size_timeout_id) {
-		g_source_remove (window->size_timeout_id);
-	}
+	GList *a;
 
-	window->size_timeout_id = g_timeout_add_seconds (1,
-							 (GSourceFunc) main_window_configure_event_timeout_cb,
-							 window);
+	a = tp_account_manager_get_valid_accounts (manager);
 
-	return FALSE;
+	gtk_action_set_sensitive (window->view_history,
+		g_list_length (a) > 0);
+
+	g_list_free (a);
 }
 
 static void
-main_window_account_created_or_deleted_cb (EmpathyAccountManager  *manager,
-					   EmpathyAccount         *account,
-					   EmpathyMainWindow      *window)
+main_window_account_validity_changed_cb (TpAccountManager *manager,
+					 TpAccount *account,
+					 gboolean valid,
+					 EmpathyMainWindow *window)
 {
-	gtk_action_set_sensitive (window->view_history,
-		empathy_account_manager_get_count (manager) > 0);
+	main_window_account_removed_cb (manager, account, window);
 }
 
 static void
@@ -1147,7 +1138,7 @@ main_window_connection_items_setup (EmpathyMainWindow *window,
 {
 	GList         *list;
 	GObject       *action;
-	gint           i;
+	guint          i;
 	const gchar *actions_connected[] = {
 		"room",
 		"chat_new_message",
@@ -1166,12 +1157,47 @@ main_window_connection_items_setup (EmpathyMainWindow *window,
 GtkWidget *
 empathy_main_window_get (void)
 {
-  return window != NULL ? window->window : NULL;
+  return main_window != NULL ? main_window->window : NULL;
+}
+
+static void
+account_manager_prepared_cb (GObject *source_object,
+			     GAsyncResult *result,
+			     gpointer user_data)
+{
+	GList *accounts, *j;
+	TpAccountManager *manager = TP_ACCOUNT_MANAGER (source_object);
+	EmpathyMainWindow *window = user_data;
+	GError *error = NULL;
+
+	if (!tp_account_manager_prepare_finish (manager, result, &error)) {
+		DEBUG ("Failed to prepare account manager: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	accounts = tp_account_manager_get_valid_accounts (window->account_manager);
+	for (j = accounts; j != NULL; j = j->next) {
+		TpAccount *account = TP_ACCOUNT (j->data);
+
+		g_signal_connect (account, "status-changed",
+				  G_CALLBACK (main_window_connection_changed_cb),
+				  window);
+	}
+
+	main_window_update_status (window);
+
+	/* Disable the "Previous Conversations" menu entry if there is no account */
+	gtk_action_set_sensitive (window->view_history,
+		g_list_length (accounts) > 0);
+
+	g_list_free (accounts);
 }
 
 GtkWidget *
 empathy_main_window_show (void)
 {
+	EmpathyMainWindow        *window;
 	EmpathyContactList       *list_iface;
 	EmpathyContactMonitor    *monitor;
 	GtkBuilder               *gui;
@@ -1182,16 +1208,16 @@ empathy_main_window_show (void)
 	GtkAction                *show_map_widget;
 	GtkToolItem              *item;
 	gboolean                  show_offline;
-	gint                      x, y, w, h;
 	gchar                    *filename;
 	GSList                   *l;
 
-	if (window) {
-		empathy_window_present (GTK_WINDOW (window->window), TRUE);
-		return window->window;
+	if (main_window) {
+		empathy_window_present (GTK_WINDOW (main_window->window), TRUE);
+		return main_window->window;
 	}
 
-	window = g_new0 (EmpathyMainWindow, 1);
+	main_window = g_new0 (EmpathyMainWindow, 1);
+	window = main_window;
 
 	/* Set up interface */
 	filename = empathy_file_lookup ("empathy-main-window.ui", "src");
@@ -1216,7 +1242,6 @@ empathy_main_window_show (void)
 
 	empathy_builder_connect (gui, window,
 			      "main_window", "destroy", main_window_destroy_cb,
-			      "main_window", "configure_event", main_window_configure_event_cb,
 			      "chat_quit", "activate", main_window_chat_quit_cb,
 			      "chat_new_message", "activate", main_window_chat_new_message_cb,
 			      "view_history", "activate", main_window_view_history_cb,
@@ -1248,11 +1273,10 @@ empathy_main_window_show (void)
 	gtk_action_set_visible (show_map_widget, FALSE);
 #endif
 
-	window->account_manager = empathy_account_manager_dup_singleton ();
+	window->account_manager = tp_account_manager_dup ();
 
-	g_signal_connect (window->account_manager,
-			  "account-connection-changed",
-			  G_CALLBACK (main_window_connection_changed_cb), window);
+	tp_account_manager_prepare_async (window->account_manager, NULL,
+					  account_manager_prepared_cb, window);
 
 	window->errors = g_hash_table_new_full (g_direct_hash,
 						g_direct_equal,
@@ -1323,23 +1347,7 @@ empathy_main_window_show (void)
 	main_window_accels_load ();
 
 	/* Set window size. */
-	empathy_geometry_load (GEOMETRY_NAME, &x, &y, &w, &h);
-
-	if (w >= 1 && h >= 1) {
-		/* Use the defaults from the ui file if we
-		 * don't have good w, h geometry.
-		 */
-		DEBUG ("Configuring window default size w:%d, h:%d", w, h);
-		gtk_window_set_default_size (GTK_WINDOW (window->window), w, h);
-	}
-
-	if (x >= 0 && y >= 0) {
-		/* Let the window manager position it if we
-		 * don't have good x, y coordinates.
-		 */
-		DEBUG ("Configuring window default position x:%d, y:%d", x, y);
-		gtk_window_move (GTK_WINDOW (window->window), x, y);
-	}
+	empathy_geometry_bind (GTK_WINDOW (window->window), GEOMETRY_NAME);
 
 	/* Enable event handling */
 	window->event_manager = empathy_event_manager_dup_singleton ();
@@ -1350,13 +1358,12 @@ empathy_main_window_show (void)
 			  G_CALLBACK (main_window_event_removed_cb),
 			  window);
 
-	g_signal_connect (window->account_manager, "account-created",
-			  G_CALLBACK (main_window_account_created_or_deleted_cb),
+	g_signal_connect (window->account_manager, "account-validity-changed",
+			  G_CALLBACK (main_window_account_validity_changed_cb),
 			  window);
-	g_signal_connect (window->account_manager, "account-deleted",
-			  G_CALLBACK (main_window_account_created_or_deleted_cb),
+	g_signal_connect (window->account_manager, "account-removed",
+			  G_CALLBACK (main_window_account_removed_cb),
 			  window);
-	main_window_account_created_or_deleted_cb (window->account_manager, NULL, window);
 
 	l = empathy_event_manager_get_events (window->event_manager);
 	while (l) {
@@ -1402,10 +1409,6 @@ empathy_main_window_show (void)
 						 EMPATHY_PREFS_UI_SHOW_AVATARS,
 						 window);
 
-	main_window_update_status (window, window->account_manager);
-
-	gtk_widget_show (window->window);
-
 	return window->window;
 }
 
